import json
import yaml
import os


def _load_json(path):
    with open(path, 'r') as f:
        return json.load(f)


def _dump_json(path, obj):
    with open(path, 'w') as f:
        json.dump(obj, f, indent=4)


def _load_yaml(path):
    with open(path, 'r') as f:
        return yaml.safe_load(f)


def _dump_yaml(path, obj):
    with open(path, 'w') as f:
        yaml.dump(obj, f)


LOADERS = {
    '.json': _load_json,
    '.yaml': _load_yaml,
    '.yml': _load_yaml
}

DUMPERS = {
    '.json': _dump_json,
    '.yaml': _dump_yaml,
    '.yml': _dump_yaml
}


def load(path):
    extension = os.path.splitext(path)[1].lower()
    if extension in LOADERS:
        return LOADERS[extension](path)
    else:
        raise TypeError(f'Unsupported extension {extension}')


def dump(path, obj):
    extension = os.path.splitext(path)[1].lower()
    if extension in DUMPERS:
        return DUMPERS[extension](path, obj)
    else:
        raise TypeError(f'Unsupported extension {extension}')



import os
from addict import Dict
import itertools
import copy

from .io import dump as _dump
from .io import load as _load


class Config(Dict):
    def __missing__(self, key):
        raise KeyError(key)

    def dump_to_file(self, path):
        _dump(path, self.to_dict())

    @classmethod
    def load_from_file(cls, path, default=None, default_search_paths=None,
                       expand=False, expand_suffix='_expand'):
        current_config = _load(path)
        if 'default' in current_config:
            default = current_config['default']
        if 'default_search_paths' in current_config:
            default_search_paths = copy.deepcopy(
                current_config['default_search_paths'])
        elif default_search_paths is None:
            default_search_paths = []
        default_search_paths.insert(0, os.path.dirname(path))
        if 'expand' in current_config:
            expand = current_config['expand']
        if 'expand_suffix' in current_config:
            expand_suffix = current_config['expand_suffix']

        if default is not None:
            default_config = None
            for path in default_search_paths:
                default_path = os.path.join(path, default)
                if os.path.exists(default_path):
                    default_config = cls.load_from_file(default_path)
                    break
            if default_config is None:
                raise ValueError(
                    f"File {default} not found in {default_search_paths}")
        else:
            default_config = {}
        default_config.update(current_config)
        default_config = cls(default_config)

        if expand:
            default_configs = default_config.expand(
                expand_suffix=expand_suffix)
        else:
            default_configs = default_config

        return default_configs

    def expand(self, expand_suffix='_expand'):
        configs = self._expand(self, expand_suffix)
        configs = [self.__class__(v) for v in configs]
        return configs

    @classmethod
    def _expand(cls, item, expand_suffix):
        if isinstance(item, cls):
            keys = list(item.keys())
            values = list(item.values())
            for i in range(len(keys)):
                expand_key = keys[i] + expand_suffix
                if expand_key in item and item[expand_key]:
                    if isinstance(values[i], list):
                        expanded = [cls._expand(v, expand_suffix)
                                    for v in values[i]]
                        values[i] = [v for sub in expanded for v in sub]
                    else:
                        raise ValueError(
                            f'The value of {keys[i]} should be a list in order'
                            f' to be expanded')
                else:
                    values[i] = cls._expand(values[i], expand_suffix)
            new_items = [dict(zip(keys, pairs))
                         for pairs in itertools.product(*values)]
            return new_items
        elif isinstance(item, (list, tuple)):
            values = [cls._expand(v, expand_suffix) for v in item]
            new_items = [type(item)(pairs)
                         for pairs in itertools.product(*values)]
            return new_items
        else:
            return [item]



from pathlib import Path
from setuptools import setup

SHORT = 'Advanced config reading/writing/parsing for yaml/json configs.'

this_directory = Path(__file__).parent
LONG = (this_directory / "README.md").read_text()

setup(
    name='config_io',
    version='0.3.0',
    packages=['config_io'],
    author='Zinan Lin',
    author_email='linzinan1995@gmail.com',
    description=SHORT,
    long_description=LONG,
    long_description_content_type='text/markdown',
    keywords='config yaml json python addict',
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    platforms='Any',
    license='MIT License',
    url='https://github.com/fjxmlzn/config_io',
    install_requires=['addict', 'pyyaml'],
    package_data={'': ['LICENSE']}
)
